@@ -1,5 +1,5 @@
 /* Basic C++ demangling support for GDB.
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef GDB_DEMANGLE_H
-#define GDB_DEMANGLE_H
+#ifndef GDB_GDB_DEMANGLE_H
+#define GDB_GDB_DEMANGLE_H
 
 /* True means that encoded C++/ObjC names should be printed out in their
    C++/ObjC form rather than raw.  */
@@ -29,6 +29,6 @@ extern bool demangle;
 extern bool asm_demangle;
 
 /* Check if a character is one of the commonly used C++ marker characters.  */
-extern int is_cplus_marker (int);
+extern bool is_cplus_marker (int);
 
-#endif /* GDB_DEMANGLE_H */
+#endif /* GDB_GDB_DEMANGLE_H */
