@@ -571,8 +571,8 @@ static const yytype_uint16 yyrline[] =
      372,   372,   393,   397,   401,   405,   409,   413,   417,   424,
      431,   438,   445,   452,   459,   463,   467,   471,   475,   482,
      489,   497,   509,   518,   521,   547,   556,   560,   582,   609,
-     628,   642,   656,   670,   671,   682,   740,   751,   755,   757,
-     759,   764,   774,   775,   776,   777,   780,   781
+     628,   642,   656,   670,   671,   682,   739,   750,   754,   756,
+     758,   763,   773,   774,   775,   776,   779,   780
 };
 #endif
 
@@ -2000,7 +2000,7 @@ yyreduce:
 			    {
 			      std::string copy = copy_name ((yyvsp[0].ssym).stoken);
 			      struct symtab *tem =
-				  lookup_symtab (copy.c_str ());
+				  lookup_symtab (current_program_space, copy.c_str ());
 			      if (tem)
 				(yyval.bval) = (tem->compunit ()->blockvector ()
 				      ->static_block ());
@@ -2112,16 +2112,15 @@ yyreduce:
 			    }
 			  else
 			    {
-			      struct bound_minimal_symbol msymbol;
 			      std::string arg = copy_name ((yyvsp[0].ssym).stoken);
 
-			      msymbol =
-				lookup_bound_minimal_symbol (arg.c_str ());
+			      bound_minimal_symbol msymbol
+				= lookup_minimal_symbol (current_program_space, arg.c_str ());
 			      if (msymbol.minsym != NULL)
 				pstate->push_new<var_msym_value_operation>
 				  (msymbol);
-			      else if (!have_full_symbols ()
-				       && !have_partial_symbols ())
+			      else if (!have_full_symbols (current_program_space)
+				       && !have_partial_symbols (current_program_space))
 				error (_("No symbol table is loaded.  "
 				       "Use the \"file\" command."));
 			      else
@@ -2129,65 +2128,65 @@ yyreduce:
 				       arg.c_str ());
 			    }
 			}
-#line 2134 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2133 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 68:
-#line 756 "p-exp.y" /* yacc.c:1646  */
+#line 755 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = lookup_pointer_type ((yyvsp[0].tval)); }
-#line 2140 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2139 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 69:
-#line 758 "p-exp.y" /* yacc.c:1646  */
+#line 757 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval) = (yyvsp[0].tsym).type; }
-#line 2146 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2145 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 70:
-#line 760 "p-exp.y" /* yacc.c:1646  */
+#line 759 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval)
 			    = lookup_struct (copy_name ((yyvsp[0].sval)).c_str (),
 					     pstate->expression_context_block);
 			}
-#line 2155 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2154 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 71:
-#line 765 "p-exp.y" /* yacc.c:1646  */
+#line 764 "p-exp.y" /* yacc.c:1646  */
     { (yyval.tval)
 			    = lookup_struct (copy_name ((yyvsp[0].sval)).c_str (),
 					     pstate->expression_context_block);
 			}
-#line 2164 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2163 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 72:
-#line 774 "p-exp.y" /* yacc.c:1646  */
+#line 773 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].ssym).stoken; }
-#line 2170 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2169 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 73:
-#line 775 "p-exp.y" /* yacc.c:1646  */
+#line 774 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].ssym).stoken; }
-#line 2176 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2175 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 74:
-#line 776 "p-exp.y" /* yacc.c:1646  */
+#line 775 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].tsym).stoken; }
-#line 2182 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2181 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
   case 75:
-#line 777 "p-exp.y" /* yacc.c:1646  */
+#line 776 "p-exp.y" /* yacc.c:1646  */
     { (yyval.sval) = (yyvsp[0].ssym).stoken; }
-#line 2188 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2187 "p-exp.c.tmp" /* yacc.c:1646  */
     break;
 
 
-#line 2192 "p-exp.c.tmp" /* yacc.c:1646  */
+#line 2191 "p-exp.c.tmp" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2415,7 +2414,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 791 "p-exp.y" /* yacc.c:1906  */
+#line 790 "p-exp.y" /* yacc.c:1906  */
 
 
 /* Take care of parsing a number (anything that starts with a digit).
@@ -3149,7 +3148,7 @@ yylex (void)
        no psymtabs (coff, xcoff, or some future change to blow away the
        psymtabs once once symbols are read).  */
     if ((sym && sym->aclass () == LOC_BLOCK)
-	|| lookup_symtab (tmp.c_str ()))
+	|| lookup_symtab (current_program_space, tmp.c_str ()))
       {
 	yylval.ssym.sym.symbol = sym;
 	yylval.ssym.sym.block = NULL;
