@@ -1,5 +1,5 @@
 /* UI_FILE - a generic STDIO like output stream.
-   Copyright (C) 1999-2022 Free Software Foundation, Inc.
+   Copyright (C) 1999-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef UI_FILE_H
-#define UI_FILE_H
+#ifndef GDB_UI_FILE_H
+#define GDB_UI_FILE_H
 
 #include <string>
 #include "ui-style.h"
@@ -30,6 +30,8 @@ public:
   ui_file ();
   virtual ~ui_file () = 0;
 
+  ui_file (ui_file &&other) = default;
+
   /* Public non-virtual API.  */
 
   void printf (const char *, ...) ATTRIBUTE_PRINTF (2, 3);
@@ -80,10 +82,10 @@ public:
   virtual bool isatty ()
   { return false; }
 
-  /* true indicates terminal output behaviour such as cli_styling.
+  /* true indicates terminal output behavior such as cli_styling.
      This default implementation indicates to do terminal output
-     behaviour if the UI_FILE is a tty.  A derived class can override
-     TERM_OUT to have cli_styling behaviour without being a tty.  */
+     behavior if the UI_FILE is a tty.  A derived class can override
+     TERM_OUT to have cli_styling behavior without being a tty.  */
   virtual bool term_out ()
   { return isatty (); }
 
@@ -169,14 +171,16 @@ class string_file : public ui_file
 {
 public:
   /* Construct a string_file to collect 'raw' output, i.e. without
-     'terminal' behaviour such as cli_styling.  */
+     'terminal' behavior such as cli_styling.  */
   string_file () : m_term_out (false) {};
-  /* If TERM_OUT, construct a string_file with terminal output behaviour
+  /* If TERM_OUT, construct a string_file with terminal output behavior
      such as cli_styling)
      else collect 'raw' output like the previous constructor.  */
   explicit string_file (bool term_out) : m_term_out (term_out) {};
   ~string_file () override;
 
+  string_file (string_file &&other) = default;
+
   /* Override ui_file methods.  */
 
   void write (const char *buf, long length_buf) override;
@@ -463,4 +467,4 @@ private:
   bool m_needs_timestamp = true;
 };
 
-#endif
+#endif /* GDB_UI_FILE_H */
