@@ -1,6 +1,6 @@
 /* Blackfin Watchpoint (WP) model.
 
-   Copyright (C) 2010-2019 Free Software Foundation, Inc.
+   Copyright (C) 2010-2024 Free Software Foundation, Inc.
    Contributed by Analog Devices, Inc.
 
    This file is part of simulators.
@@ -18,7 +18,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "config.h"
+/* This must come before any other includes.  */
+#include "defs.h"
 
 #include "sim-main.h"
 #include "devices.h"
@@ -83,7 +84,7 @@ bfin_wp_io_write_buffer (struct hw *me, const void *source, int space,
 
   value = dv_load_4 (source);
   mmr_off = addr - wp->base;
-  valuep = (void *)((unsigned long)wp + mmr_base() + mmr_off);
+  valuep = (void *)((uintptr_t)wp + mmr_base() + mmr_off);
 
   HW_TRACE_WRITE ();
 
@@ -123,7 +124,7 @@ bfin_wp_io_read_buffer (struct hw *me, void *dest, int space,
     return 0;
 
   mmr_off = addr - wp->base;
-  valuep = (void *)((unsigned long)wp + mmr_base() + mmr_off);
+  valuep = (void *)((uintptr_t)wp + mmr_base() + mmr_off);
 
   HW_TRACE_READ ();
 
